/*
 * File:		dryice_functinos.c
 * Purpose:		Functions used for DryIce module.
 *
 */

#include "common.h"
#include "dryice_functions.h"
#include "smc.h"
#include "uart.h"

/********************************************************************/
////////////////////////////////////////////////////////////////////////////////
// Coverts pin number to corresponding register bit location
int power(int n)
{
  int i, p;
  p = 1;
  for (i= 1; i <= n; ++i)
    p = p * 2;
  return p;
}

////////////////////////////////////////////////////////////////////////////////
void dryice_prescaler_config(int desired_prescaler_value)
{
  int current_prescaler_value;

  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Prescaler Config\n");
  printf("-------------------------------------------\n");
  printf("\n");
  
   if ((DRY_RAC & DRY_RAC_CRR_MASK)>>DRY_RAC_CRR_SHIFT == 0)
  {
    printf("Cannot read the DryIce prescaler value. The DRY_CR register's read access is blocked. Remove the read access block and try again.\n");
    return;
  } 
  
  if (((DRY_LR & DRY_LR_CRL_MASK)>>DRY_LR_CRL_SHIFT == 0) || ((DRY_WAC & DRY_WAC_CRW_MASK)>>DRY_WAC_CRW_SHIFT == 0))
  {
    printf("Cannot write the DryIce prescaler value. The DRY_CR register is locked or its write access is blocked. Remove the lock or its write access block and try again.\n");
    printf("DRY_LR is \t%08X\n",DRY_LR);
    printf("DRY_WAC is \t%08X\n",DRY_WAC);
    printf("\n");
    return;    
  }    
  
  //Cannot write prescaler if DryIce clocks are enabled
  if ((DRY_CR & DRY_CR_DEN_MASK) != 0x0)
  {
    printf("DryIce clock and prescaler are enabled. Please disable them first before specifying a new prescaler value.\n\n");
    return;
  }
 
  if ((desired_prescaler_value < 0x0) || (desired_prescaler_value > 0x7FFF))
  {
    printf("Invalid prescaler value:\n");
    printf("Valid 'ps' syntax:\n");
    printf("DRYICE_CMD> ps  <0x0..0x7FFF = prescaler value>\n\n");
    printf("\n");    
    return;    
  }
  
  DRY_CR &= ~DRY_CR_DPR(0x7FFF);
  DRY_CR |= DRY_CR_DPR(desired_prescaler_value);
  
  current_prescaler_value = (DRY_CR & DRY_CR_DPR_MASK);
  printf("\nNew prescaler value is \t%i\n",current_prescaler_value>>DRY_CR_DPR_SHIFT);
  printf("\n");  
  return;
}

////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_polarity_config(int pin_number, int desired_pin_polarity)
{
  int32 corresponding_tamper_pin;  
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Polarity Config Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_PPR_MASK)>>DRY_RAC_PPR_SHIFT == 0)
  {
    printf("Cannot read the pin polarity setting. The DRY_PPR register's read access is blocked. Remove the read access block and try again.\n");
    return;
  }         

  if (((DRY_LR & DRY_LR_PPL_MASK)>>DRY_LR_PPL_SHIFT == 0) || ((DRY_WAC & DRY_WAC_PPW_MASK)>>DRY_WAC_PPW_SHIFT == 0))
  {
    printf("Cannot write the pin polarity setting. The DRY_PPR register is locked or its write access is blocked. Remove the lock or its write access block and try again.\n");
    return;    
  }   
  
  if (((DRY_LR & DRY_LR_PPL_MASK) == 0x0) || ((DRY_WAC & DRY_WAC_PPW_MASK) == 0x0) || ((DRY_RAC & DRY_RAC_PPR_MASK) == 0x0))
  {
    printf("Cannot write DRY_PPR because it is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    return;
  }
  
  
  //Determine value to write into PPR[TPP]
  corresponding_tamper_pin = power(pin_number);
 
  if ((pin_number < 0) || (pin_number > 7))
  {
    printf("Invalid tamper pin source:\n");    
    return;
  }
  
  if ((desired_pin_polarity < 0) || (desired_pin_polarity > 1))
  {
    printf("Invalid pin polarity:\n"); 
    return;
  }
  

  if (desired_pin_polarity == 0) {
    DRY_PPR &= ~DRY_PPR_TPP(corresponding_tamper_pin);
    printf("Tamper Pin %i expected value is not inverted.\n",pin_number);
  }
  else
  {
    DRY_PPR |= DRY_PPR_TPP(corresponding_tamper_pin);
    printf("Tamper Pin %i expected value is inverted.\n",pin_number);            
  }

  return;
}// end dryice_tamper_pin_polarity_config
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_direction_config(int pin_number, int desired_pin_direction)
{
  int32 corresponding_tamper_pin;

  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Direction Config Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_PDR_MASK)>>DRY_RAC_PDR_SHIFT == 0)
  {
    printf("Cannot read the pin direction. The DRY_PDR register's read access is blocked. Remove the read access block and try again.\n");
    return;
  }          

  if (((DRY_LR & DRY_LR_PDL_MASK)>>DRY_LR_PDL_SHIFT == 0) || ((DRY_WAC & DRY_WAC_PDW_MASK)>>DRY_WAC_PDW_SHIFT == 0))
  {
    printf("Cannot write the pin direction. The DRY_PDR register is locked or its write access is blocked. Remove the lock or its write access block and try again.\n");
    return;    
  }  
 
  //Determine value to write into register based on pin_number
  corresponding_tamper_pin = power(pin_number);  
 
  if ((pin_number < 0) || (pin_number > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if ((desired_pin_direction < 0) || (desired_pin_direction > 1))
  {
    printf("Invalid pin direction:\n");
    return;
  }
  
  if (desired_pin_direction == 0) {
    DRY_PDR &= ~DRY_PDR_TPD(corresponding_tamper_pin);
    printf("Tamper Pin %i pin direction set to input.\n",pin_number);
  }
  else
  {
    DRY_PDR |= DRY_PDR_TPD(corresponding_tamper_pin);
    printf("Tamper Pin %i pin direction set to output and drives inverse of expected value.\n",pin_number);
  }
  return;
}// end dryice_tamper_pin_direction_config
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_glitch_filter_width_config(int tamper_pin, int desired_glitch_filter_width)
{
  unsigned char current_glitch_filter_width;

  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Glitch Width Config Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_GFR_MASK) != DRY_RAC_GFR_MASK)
  {
    printf("WARNING: Some PGFRx registers are blocked from reading.\n");
    printf("DRY_RAC is \t%08X\n",DRY_RAC);
    printf("\n");
  }    
  
 
  if ((tamper_pin < 0) || (tamper_pin > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if ((desired_glitch_filter_width < 0) || (desired_glitch_filter_width > 63))
  {
    printf("Invalid glitch filter width:\n");
    return;
  }
  
  switch (tamper_pin)
  {
  case 0:
    printf("DRY_PGFR0 is %08X\n",DRY_PGFR0); 
    if ((DRY_PGFR0 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR0 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 0 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR0 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR0 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 0 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    break;
  case 1:
    printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);        
    if ((DRY_PGFR1 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR1 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 1 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR1 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR1 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 1 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);    
    break;
  case 2:
    printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);        
    if ((DRY_PGFR2 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR2 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 2 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR2 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR2 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 2 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);    
    break;
  case 3:
    printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);        
    if ((DRY_PGFR3 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR3 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 3 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);        
    DRY_PGFR3 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR3 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 3 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);    
    break;
  case 4:
    printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);        
    if ((DRY_PGFR4 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR4 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 4 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR4 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR4 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 4 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);    
    break;
  case 5:
    printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);        
    if ((DRY_PGFR5 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }    
    current_glitch_filter_width = (DRY_PGFR5 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 5 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR5 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR5 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 5 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);    
    break;
  case 6:
    printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);     
    if ((DRY_PGFR6 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR6 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 6 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR6 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR6 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 6 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);    
    break;
  case 7:
    printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);        
    if ((DRY_PGFR7 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR7 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 7 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR7 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR7 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 7 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);    
  }
  return;
}// end dryice_tamper_pin_glitch_filter_width_config
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_glitch_filter_prescaler_config(int tamper_pin, int desired_glitch_filter_prescaler)
{
  int32 corresponding_tamper_pin;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Glitch Filter Prescaler Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_GFR_MASK) != DRY_RAC_GFR_MASK)
  {
    printf("WARNING: Some PGFRx registers are blocked from reading.\n");
    printf("DRY_RAC is \t%08X\n",DRY_RAC);
    printf("\n");
  }  

  corresponding_tamper_pin = power(tamper_pin);
  
  if ((tamper_pin < 0) || (tamper_pin > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if (((DRY_LR & DRY_LR_GFL(corresponding_tamper_pin))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0) || ((DRY_WAC & DRY_WAC_GFW(corresponding_tamper_pin))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0) || ((DRY_RAC & DRY_RAC_GFR(corresponding_tamper_pin))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
  {
    printf("Cannot write DRY_PGFR%i because it is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n",tamper_pin);
    return;
  }      
  
  if ((desired_glitch_filter_prescaler != 0) && (desired_glitch_filter_prescaler != 1))
  {
    printf("Invalid glitch filter source value:\n");
    return;
  }
  
  switch (tamper_pin)
  {
  case 0:
    if ((DRY_PGFR0 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }    
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 0 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 0 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    break;
  case 1:
    if ((DRY_PGFR1 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }    
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 1 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 1 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    break;
  case 2:
    if ((DRY_PGFR2 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 2 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 2 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    break;
  case 3:
    if ((DRY_PGFR3 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 3 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 3 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    break;
  case 4:
    if ((DRY_PGFR4 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 4 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 4 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    break;
  case 5:
    if ((DRY_PGFR5 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 5 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 5 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    break;
  case 6:
    if ((DRY_PGFR6 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 6 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 6 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    break;
  case 7:
    if ((DRY_PGFR7 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      printf("\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 7 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 7 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
  }   
  return;
}// end dryice_tamper_pin_glitch_filter_prescaler_config
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_glitch_filter_config(int tamper_pin, int desired_glitch_filter_setting)
{
  int32 corresponding_tamper_pin;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Glitch Filter Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_GFR_MASK) != DRY_RAC_GFR_MASK)
  {
    printf("WARNING: Some PGFRx registers are blocked from reading.\n");
    printf("DRY_RAC is \t%08X\n",DRY_RAC);
    printf("\n");
  }  
  
  corresponding_tamper_pin = power(tamper_pin);
  
  if ((tamper_pin < 0) || (tamper_pin > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if (((DRY_LR & DRY_LR_GFL(corresponding_tamper_pin))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0) || ((DRY_WAC & DRY_WAC_GFW(corresponding_tamper_pin))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0) || ((DRY_RAC & DRY_RAC_GFR(corresponding_tamper_pin))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
  {
    printf("Cannot write DRY_PGFR%i because it is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n",tamper_pin);
    printf("DRY_LR is \t%08X\n",DRY_LR);
    printf("DRY_WAC is \t%08X\n",DRY_WAC);
    printf("DRY_RAC is \t%08X\n",DRY_RAC);    
    printf("\n");
    return;
  }  
  
 
  if ((desired_glitch_filter_setting != 0) && (desired_glitch_filter_setting != 1))
  {
    printf("Invalid glitch filter enable/disable switch:\n");
    return;
  }

  //DryIce prescaler and clock must be enabled first before glitch filter can be enabled  
  if (((DRY_CR & DRY_CR_DEN_MASK) != DRY_CR_DEN_MASK) && (desired_glitch_filter_setting == 1))
  {
    printf("DryIce prescaler and clock is disabled. Please enable it first before enabling the glitch filter.\n");
    printf("\n");
    return;
  }
  
  switch (tamper_pin)
  {
  case 0:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 0 glitch filter disabled.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 0 glitch filter enabled.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    break;
  case 1:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 1 glitch filter disabled.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 1 glitch filter enabled.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    break;
  case 2:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 2 glitch filter disabled.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 2 glitch filter enabled.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    break;
  case 3:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 3 glitch filter disabled.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 3 glitch filter enabled.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    break;
  case 4:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 4 glitch filter disabled.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 4 glitch filter enabled.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    break;
  case 5:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 5 glitch filter disabled.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 5 glitch filter enabled.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    break;
  case 6:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 6 glitch filter disabled.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 6 glitch filter enabled.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    break;
  case 7:
    if (desired_glitch_filter_setting == 0) {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 7 glitch filter disabled.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 7 glitch filter enabled.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
  }
  return;
}// end dryice_tamper_pin_glitch_filter_config
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_expect_config(int tamper_pin, int expect_value)
{
  int32 corresponding_tamper_pin;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Expect Value Config Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_RAC & DRY_RAC_GFR_MASK) != DRY_RAC_GFR_MASK)
  {
    printf("WARNING: Some PGFRx registers are blocked from reading.\n");
    printf("DRY_RAC is \t%08X\n",DRY_RAC);
    printf("\n");
  }  
   
  corresponding_tamper_pin = power(tamper_pin);
 
  if ((tamper_pin < 0) || (tamper_pin > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if (((DRY_LR & DRY_LR_GFL(corresponding_tamper_pin))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0) || ((DRY_WAC & DRY_WAC_GFW(corresponding_tamper_pin))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0) || ((DRY_RAC & DRY_RAC_GFR(corresponding_tamper_pin))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
  {
    printf("Cannot write DRY_PGFRx because it is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    printf("DRY_LR is \t%08X\n",DRY_LR);
    printf("DRY_WAC is \t%08X\n",DRY_WAC);
    printf("DRY_RAC is \t%08X\n",DRY_RAC);    
    printf("\n");
    return;
  }  
  
  if ((expect_value < 0) || (expect_value > 3))
  {
    printf("Invalid expect value:\n");
    return;
  }
  
  switch (tamper_pin)
  {
  case 0:
    if (expect_value == 0) {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 0 expected value is logic 0.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= 0xFFFCFFFF;
      DRY_PGFR0 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 0 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= 0xFFFCFFFF;
      DRY_PGFR0 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 0 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= 0xFFFCFFFF;
      DRY_PGFR0 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 0 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }    
    break;
  case 1:
    if (expect_value == 0) {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 1 expected value is logic 0.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= 0xFFFCFFFF;
      DRY_PGFR1 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 1 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= 0xFFFCFFFF;
      DRY_PGFR1 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 1 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= 0xFFFCFFFF;
      DRY_PGFR1 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 1 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }    
    break;
  case 2:
    if (expect_value == 0) {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 2 expected value is logic 0.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= 0xFFFCFFFF;
      DRY_PGFR2 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 2 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= 0xFFFCFFFF;
      DRY_PGFR2 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 2 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= 0xFFFCFFFF;
      DRY_PGFR2 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 2 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }    
    break;
  case 3:
    if (expect_value == 0) {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 3 expected value is logic 0.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= 0xFFFCFFFF;
      DRY_PGFR3 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 0 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= 0xFFFCFFFF;
      DRY_PGFR3 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 3 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= 0xFFFCFFFF;
      DRY_PGFR3 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 3 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }    
    break;
  case 4:
    if (expect_value == 0) {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 4 expected value is logic 0.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= 0xFFFCFFFF;
      DRY_PGFR4 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 4 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= 0xFFFCFFFF;
      DRY_PGFR4 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 4 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= 0xFFFCFFFF;
      DRY_PGFR4 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 4 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }    
    break;
  case 5:
    if (expect_value == 0) {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 5 expected value is logic 0.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= 0xFFFCFFFF;
      DRY_PGFR5 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 5 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= 0xFFFCFFFF;
      DRY_PGFR5 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 5 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= 0xFFFCFFFF;
      DRY_PGFR5 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 5 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }    
    break;
  case 6:
    if (expect_value == 0) {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 6 expected value is logic 0.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= 0xFFFCFFFF;
      DRY_PGFR6 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 6 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= 0xFFFCFFFF;
      DRY_PGFR6 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 6 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= 0xFFFCFFFF;
      DRY_PGFR6 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 6 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }    
    break;
  case 7:
    if (expect_value == 0) {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= ~DRY_PGFR_TPEX(3);
      printf("Tamper Pin 7 expected value is logic 0.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else if (expect_value == 1)
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= 0xFFFCFFFF;
      DRY_PGFR7 |= DRY_PGFR_TPEX(1);
      printf("Tamper Pin 7 expected value is active tamper 0 output.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else if (expect_value == 2)
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= 0xFFFCFFFF;
      DRY_PGFR7 |= DRY_PGFR_TPEX(2);      
      printf("Tamper Pin 7 expected value is active tamper 1 output.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else if (expect_value == 3)
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= 0xFFFCFFFF;
      DRY_PGFR7 |= DRY_PGFR_TPEX(3);      
      printf("Tamper Pin 7 expected value is active tamper 0 output XOR'ed with active tamper 1 output.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
  }
  return;
}// end dryice_tamper_pin_expect_config
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_tamper_pin_pull_config(int tamper_pin, int desired_pin_pull_setting)
{

  int32 corresponding_tamper_pin;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Pull Resistor Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");

  corresponding_tamper_pin = power(tamper_pin);
  
  
  if ((tamper_pin < 0) || (tamper_pin > 7))
  {
    printf("Invalid tamper pin source:\n");
    return;
  }
  
  if (((DRY_LR & DRY_LR_GFL(corresponding_tamper_pin))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0) || ((DRY_WAC & DRY_WAC_GFW(corresponding_tamper_pin))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0) || ((DRY_RAC & DRY_RAC_GFR(corresponding_tamper_pin))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
  {
    printf("Cannot write DRY_PGFR%i because it is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n",tamper_pin);
    printf("DRY_LR is \t%08X\n",DRY_LR);
    printf("DRY_WAC is \t%08X\n",DRY_WAC);
    printf("DRY_RAC is \t%08X\n",DRY_RAC);    
    printf("\n");
    return;
  }   
  
  if ((desired_pin_pull_setting != 0) && (desired_pin_pull_setting != 1))
  {
    printf("Invalid pull enable/disable switch:\n");
    return;
  }
  
  switch (tamper_pin)
  {
  case 0:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 0 pull resistor disabled.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 0 pull resistor enabled.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    break;
  case 1:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 1 pull resistor disabled.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 1 pull resistor enabled.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    break;
  case 2:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 2 pull resistor disabled.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 2 pull resistor enabled.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);                        
    }
    break;
  case 3:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);                        
      DRY_PGFR3 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 3 pull resistor disabled.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);                        
    }
    else
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);                        
      DRY_PGFR3 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 3 pull resistor enabled.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);                        
    }
    break;
  case 4:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);                        
      DRY_PGFR4 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 4 pull resistor disabled.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);                        
    }
    else
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);                        
      DRY_PGFR4 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 4 pull resistor enabled.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);                        
    }
    break;
  case 5:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);                        
      DRY_PGFR5 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 5 pull resistor disabled.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);                        
    }
    else
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);                        
      DRY_PGFR5 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 5 pull resistor enabled.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);                  
    }
    break;
  case 6:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);                  
      DRY_PGFR6 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 6 pull resistor disabled.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);                  
    }
    else
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);                  
      DRY_PGFR6 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 6 pull resistor enabled.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);            
    }
    break;
  case 7:
    if (desired_pin_pull_setting == 0) {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);            
      DRY_PGFR7 &= ~DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 7 pull resistor disabled.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);            
    }
    else
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);            
      DRY_PGFR7 |= DRY_PGFR_TPE_MASK;
      printf("Tamper Pin 7 pull resistor enabled.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);      
    }
  }
  return;
}// end dryice_tamper_pin_pull_config
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_int_tamper_interrupt_config(int internal_tamper_source, int enable_disable)
{
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Internal Tamper Flag Interrupt Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");

  if (((DRY_LR & DRY_LR_IEL_MASK) == 0x0) || ((DRY_WAC & DRY_WAC_IEW_MASK) == 0x0) || ((DRY_RAC & DRY_RAC_IER_MASK) == 0x0))
  {
    printf("Cannot read or write the interrupt enable settings. DRY_IER is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    return;
  }
  
  
  if ((internal_tamper_source < 0) || (internal_tamper_source == 1) || (internal_tamper_source > 9))
  {
    printf("Invalid internal tamper interrupt source:\n");
    return;
  }
  
  if ((enable_disable != 0) && (enable_disable != 1))
  {
    printf("Invalid tamper interrupt enable/disable switch:\n");
    return;
  }
  
  switch (internal_tamper_source)
  {
  case 0:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_DTIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[DTF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_DTIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[DTF] is set.\n");
    }
    break;
  case 2:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_TOIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[TOF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_TOIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[TOF] is set.\n");
    }
    break;
  case 3:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_MOIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[MOF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_MOIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[MOF] is set.\n");
    }
    break;
  case 4:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_VTIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[VTF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_VTIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[VTF] is set.\n");
    }
    break;
  case 5:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_CTIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[CTF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_CTIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[CTF] is set.\n");
    }
    break;
  case 6:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_TTIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[TTF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_TTIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[TTF] is set.\n");
    }
    break;
  case 7:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_STIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[STF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_STIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[STF] is set.\n");
    }
    break;
  case 8:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_FSIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[FSF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_FSIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[FSF] is set.\n");
    }
    break;
  case 9:
    if (enable_disable == 0) {
      DRY_IER &= ~DRY_IER_TMIE_MASK;
      printf("DryIce Interrupt will not be generated if DRY_SR[TMF] is set.\n");
    }
    else
    {
      DRY_IER |= DRY_IER_TMIE_MASK;
      printf("DryIce Interrupt will be generated if DRY_SR[TMF] is set.\n");
    }
  }
  return;    
}//end dryice_int_tamper_interrupt_config
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
void dryice_pin_tamper_interrupt_config(int pin_number, int enable_disable)
{
  int32 corresponding_tamper_pin;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Tamper Flag Interrupt Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if (((DRY_LR & DRY_LR_IEL_MASK) == 0x0) || ((DRY_WAC & DRY_WAC_IEW_MASK) == 0x0) || ((DRY_RAC & DRY_RAC_IER_MASK) == 0x0))
  {
    printf("Cannot read or write interrupt enable settings. DRY_IER is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    return;
  }    
  

  corresponding_tamper_pin = power(pin_number);    
  
  if ((pin_number < 0) || (pin_number > 7))
  {
    printf("Invalid pin tamper source:\n");
    return;
  }
  
  if ((enable_disable != 0) && (enable_disable != 1))
  {
    printf("Invalid pin tamper interrupt enable/disable switch:\n");
    return;
  }
  

  if (enable_disable == 0) {
    DRY_IER &= ~DRY_IER_TPIE(corresponding_tamper_pin);
    printf("If Tamper Pin %i Flag is set, an interrupt will not be generated.\n",pin_number);
  }
  else
  {
    DRY_IER |= DRY_IER_TPIE(corresponding_tamper_pin);
    printf("If Tamper Pin %i Flag is set, an interrupt will be generated.\n",pin_number);
  }
  return;
}// end dryice_pin_tamper_interrupt_config
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
void dryice_int_tamper_config(int internal_tamper_source, int enable_disable)
{
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Internal Tamper Flag Setting DTF Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if (((DRY_LR & DRY_LR_TEL_MASK) == 0x0) || ((DRY_WAC & DRY_WAC_TEW_MASK) == 0x0) || ((DRY_RAC & DRY_RAC_TER_MASK) == 0x0))
  {
    printf("Cannot read or write tamper enable settings. DRY_TER either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    return;
  }    
  
  
  
  if ((internal_tamper_source < 2) || (internal_tamper_source > 9))
  {
    printf("Invalid internal tamper source:\n");
    return;
  }
  
  if ((enable_disable != 0) && (enable_disable != 1))
  {
    printf("Invalid tamper enable/disable switch:\n");
    return;
  }
  
  switch (internal_tamper_source)
  {
  case 2:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_TOE_MASK;
      printf("Time Overflow Flag (DRY_SR[TOF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_TOE_MASK;
      printf("Time Overflow Flag (DRY_SR[TOF]) assertion will set DRY_SR[DTF].\n");
    }
    break;
  case 3:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_MOE_MASK;
      printf("Monotonic Overflow Flag (DRY_SR[MOF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_MOE_MASK;
      printf("Monotonic Overflow Flag (DRY_SR[MOF]) assertion will set DRY_SR[DTF].\n");
    }
    break;
  case 4:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_VTE_MASK;
      printf("Disabling clock tamper circuitry.\n");            
      printf("Voltage Tamper Flag (DRY_SR[VTF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_VTE_MASK;
      printf("Enabling voltage tamper circuitry.\n");            
      printf("Voltage Tamper Flag (DRY_SR[VTF]) assertion will set DRY_SR[DTF].\n");
      printf("Please also enable clock tamper detect.\n");      
    }
    break;
  case 5:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_CTE_MASK;      
      printf("Disabling clock tamper circuitry.\n");            
      printf("Clock Tamper Flag (DRY_SR[CTF]) assertion will not set DRY_SR[DTF].\n");
      printf("Please also disable voltage and temperature tamper detect.\n");
    }
    else
    {
      DRY_TER |= DRY_TER_CTE_MASK; 
      printf("Enabling clock tamper circuitry.\n");            
      printf("Clock Tamper Flag (DRY_SR[CTF]) assertion will set DRY_SR[DTF].\n");
    }
    break;
  case 6:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_TTE_MASK;
      printf("Disabling temperature tamper circuitry.\n");
      printf("Temperature Tamper Flag (DRY_SR[TTF]) assertion will not set DRY_SR[DTF].\n");      
    }
    else
    {
      DRY_TER |= DRY_TER_TTE_MASK;      
      printf("Enabling temperature tamper circuitry.\n");      
      printf("Temperature Tamper Flag (DRY_SR[TTF]) assertion will set DRY_SR[DTF].\n");
      printf("Please also enable clock tamper detect.\n");      
    }
    break;
  case 7:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_STE_MASK;
      printf("Security Module Tamper Flag (DRY_SR[STF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_STE_MASK;
      printf("Security Module Tamper Flag (DRY_SR[STF] assertion will set DRY_SR[DTF].\n");
    }
    break;
  case 8:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_FSE_MASK;
      printf("Flash Security Flag (DRY_SR[FSF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_FSE_MASK;
      printf("Flash Security Flag (DRY_SR[FSF]) assertion will set DRY_SR[DTF].\n");
    }
    break;
  case 9:
    if (enable_disable == 0) {
      DRY_TER &= ~DRY_TER_TME_MASK;
      printf("Test Mode Flag (DRY_SR[TMF]) assertion will not set DRY_SR[DTF].\n");
    }
    else
    {
      DRY_TER |= DRY_TER_TME_MASK;
      printf("Test Mode Flag (DRY_SR[TMF]) assertion will set DRY_SR[DTF].\n");
    }
  }
  return;
}//end dryice_int_tamper_config
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
void dryice_pin_tamper_config(int pin_number, int enable_disable)
{
  int32 corresponding_tamper_pin;

  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Tamper Flag Setting DTF Enable/Disable Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if (((DRY_LR & DRY_LR_TEL_MASK) == 0x0) || ((DRY_WAC & DRY_WAC_TEW_MASK) == 0x0) || ((DRY_RAC & DRY_RAC_TER_MASK) == 0x0))
  {
    printf("Cannot read or write tamper enable settings. DRY_TER is either locked or its read or write access is blocked. Remove the lock or the read/write access block and try again.\n");
    return;
  }    
  
  //Determine value to write to TER[TPE]
  corresponding_tamper_pin = power(pin_number);    
  
  if ((pin_number < 0) || (pin_number > 7))
  {
    printf("Invalid pin tamper source:\n");
    return;
  }
  
  if ((enable_disable != 0) && (enable_disable != 1))
  {
    printf("Invalid tamper enable/disable switch:\n");
    return;
  }

  if (enable_disable == 0) {
    DRY_TER &= ~DRY_TER_TPE(corresponding_tamper_pin);
    printf("Tamper Event on Pin %i will not set either SR[TPF%i] or SR[DTF].\n",pin_number,pin_number);
  }
  else
  {
    DRY_TER |= DRY_TER_TPE(corresponding_tamper_pin);
    printf("Tamper Event on Pin %i will set both SR[TPF%i] and SR[DTF].\n",pin_number,pin_number);
  }
  return;
}// end dryice_pin_tamper_config
////////////////////////////////////////////////////////////////////////////////
void glitch_filter_prescaler_switch(int tamper_pin, int desired_glitch_filter_setting)
{
  switch (tamper_pin)
  {
  case 0:
    if (((DRY_RAC & DRY_RAC_GFR(0x1))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR0 because its read access is blocked. Remove the read access block and try again.\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x1))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because its write access is blocked. Remove the write access block and try again.\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x1))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because it is locked. Remove the lock and try again.\n");
      return;
    }        
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 0 glitch filter disabled.\n");
      printf("New DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else
    {
      printf("Current DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 0 glitch filter enabled.\n");
      printf("New DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    break;
  case 1:
    if (((DRY_RAC & DRY_RAC_GFR(0x2))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR1 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x2))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x2))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }    
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 1 glitch filter disabled.\n");
      printf("New DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else
    {
      printf("Current DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 1 glitch filter enabled.\n");
      printf("New DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    break;
  case 2:
    if (((DRY_RAC & DRY_RAC_GFR(0x4))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR2 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x4))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x4))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }    
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 2 glitch filter disabled.\n");
      printf("New DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else
    {
      printf("Current DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 2 glitch filter enabled.\n");
      printf("New DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    break;
  case 3:
    if (((DRY_RAC & DRY_RAC_GFR(0x8))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR3 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x8))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x8))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 3 glitch filter disabled.\n");
      printf("New DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else
    {
      printf("Current DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 3 glitch filter enabled.\n");
      printf("New DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    break;
  case 4:
    if (((DRY_RAC & DRY_RAC_GFR(0x10))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR4 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x10))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x10))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }             
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 4 glitch filter disabled.\n");
      printf("New DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else
    {
      printf("Current DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 4 glitch filter enabled.\n");
      printf("New DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    break;
  case 5:
    if (((DRY_RAC & DRY_RAC_GFR(0x20))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR5 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x20))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x20))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }             
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 5 glitch filter disabled.\n");
      printf("New DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else
    {
      printf("Current DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 5 glitch filter enabled.\n");
      printf("New DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    break;
  case 6:
    if (((DRY_RAC & DRY_RAC_GFR(0x40))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR6 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x40))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x40))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }             
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 6 glitch filter disabled.\n");
      printf("New DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else
    {
      printf("Current DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 6 glitch filter enabled.\n");
      printf("New DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    break;
  case 7:
    if (((DRY_RAC & DRY_RAC_GFR(0x80))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR7 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x80))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x80))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }             
    if (desired_glitch_filter_setting == 0) {
      printf("Current DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= ~DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 7 glitch filter disabled.\n");
      printf("New DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else
    {
      printf("Current DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 |= DRY_PGFR_GFE_MASK;
      printf("Tamper Pin 7 glitch filter enabled.\n");
      printf("New DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
  }        
}
////////////////////////////////////////////////////////////////////////////////
void glitch_filter_prescaler_select(int tamper_pin, int desired_glitch_filter_prescaler)
{
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Glitch Filter Clock Select Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");  
  switch (tamper_pin)
  {
  case 0:
    if (((DRY_RAC & DRY_RAC_GFR(0x1))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR0 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x1))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x1))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }        
    if ((DRY_PGFR0 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }    
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 0 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    else
    {
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
      DRY_PGFR0 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 0 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    }
    break;
  case 1:
   if (((DRY_RAC & DRY_RAC_GFR(0x2))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR1 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x2))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x2))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }    
    if ((DRY_PGFR1 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }    
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 1 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    else
    {
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
      DRY_PGFR1 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 1 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);
    }
    break;
  case 2:
    if (((DRY_RAC & DRY_RAC_GFR(0x4))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR2 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x4))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x4))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }         
    if ((DRY_PGFR2 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 2 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    else
    {
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
      DRY_PGFR2 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 2 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);
    }
    break;
  case 3:
    if (((DRY_RAC & DRY_RAC_GFR(0x8))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR3 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x8))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x8))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    
    if ((DRY_PGFR3 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 3 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    else
    {
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
      DRY_PGFR3 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 3 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);
    }
    break;
  case 4:
    if (((DRY_RAC & DRY_RAC_GFR(0x10))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR4 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x10))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x10))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }         
    if ((DRY_PGFR4 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 4 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    else
    {
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
      DRY_PGFR4 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 4 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);
    }
    break;
  case 5:
    if (((DRY_RAC & DRY_RAC_GFR(0x20))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR5 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x20))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x20))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }         
    if ((DRY_PGFR5 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 5 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    else
    {
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
      DRY_PGFR5 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 5 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);
    }
    break;
  case 6:
    if (((DRY_RAC & DRY_RAC_GFR(0x40))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR6 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x40))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x40))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }         
    if ((DRY_PGFR6 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 6 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    else
    {
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
      DRY_PGFR6 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 6 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);
    }
    break;
  case 7:
    if (((DRY_RAC & DRY_RAC_GFR(0x80))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR7 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x80))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x80))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }         
    if ((DRY_PGFR7 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter prescaler setting when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    if (desired_glitch_filter_prescaler == 0) {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 &= ~DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 7 glitch filter sourced by 512 Hz clock.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
    else
    {
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
      DRY_PGFR7 |= DRY_PGFR_GFP_MASK;
      printf("Tamper Pin 7 glitch filter sourced by 32.768 kHz clock.\n");
      printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);
    }
  }      
}
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void glitch_filter_width_select(int tamper_pin, int desired_glitch_filter_width)
{
  int current_glitch_filter_width;
  
  switch (tamper_pin)
  {
  case 0:
    if (((DRY_RAC & DRY_RAC_GFR(0x1))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR0 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x1))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x1))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR0 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }    
    printf("DRY_PGFR0 is %08X\n",DRY_PGFR0); 
    if ((DRY_PGFR0 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR0 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 0 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR0 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR0 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 0 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR0 is %08X\n",DRY_PGFR0);
    break;
  case 1:
    if (((DRY_RAC & DRY_RAC_GFR(0x2))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR1 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x2))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x2))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR1 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);        
    if ((DRY_PGFR1 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR1 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 1 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR1 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR1 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 1 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR1 is %08X\n",DRY_PGFR1);    
    break;
  case 2:
    if (((DRY_RAC & DRY_RAC_GFR(0x4))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR2 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x4))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x4))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR2 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);        
    if ((DRY_PGFR2 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR2 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 2 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);    
    DRY_PGFR2 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR2 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 2 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR2 is %08X\n",DRY_PGFR2);    
    break;
  case 3:
    if (((DRY_RAC & DRY_RAC_GFR(0x8))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR3 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x8))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x8))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR3 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);        
    if ((DRY_PGFR3 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR3 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 3 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);        
    DRY_PGFR3 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR3 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 3 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR3 is %08X\n",DRY_PGFR3);    
    break;
  case 4:
    if (((DRY_RAC & DRY_RAC_GFR(0x10))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR4 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x10))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x10))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR4 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);        
    if ((DRY_PGFR4 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR4 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 4 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR4 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR4 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 4 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR4 is %08X\n",DRY_PGFR4);    
    break;
  case 5:
    if (((DRY_RAC & DRY_RAC_GFR(0x20))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR5 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x20))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x20))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR5 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);        
    if ((DRY_PGFR5 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }    
    current_glitch_filter_width = (DRY_PGFR5 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 5 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR5 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR5 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 5 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR5 is %08X\n",DRY_PGFR5);    
    break;
  case 6:
    if (((DRY_RAC & DRY_RAC_GFR(0x40))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR6 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x40))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x40))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR6 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);     
    if ((DRY_PGFR6 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR6 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 6 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR6 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR6 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 6 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR6 is %08X\n",DRY_PGFR6);    
    break;
  case 7:
    if (((DRY_RAC & DRY_RAC_GFR(0x80))>>(DRY_RAC_GFR_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot read DRY_PGFR7 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_GFW(0x80))>>(DRY_WAC_GFW_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_GFL(0x80))>>(DRY_LR_GFL_SHIFT+tamper_pin) == 0x0))
    {
      printf("Cannot write DRY_PGFR7 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }     
    printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);        
    if ((DRY_PGFR7 & DRY_PGFR_GFE_MASK) == DRY_PGFR_GFE_MASK)
    {
      printf("Cannot change glitch filter width when glitch filter is enabled. Please disable it first.\n");
      return;
    }
    current_glitch_filter_width = (DRY_PGFR7 & DRY_PGFR_GFW_MASK);
    printf("Previous Tamper Pin 7 glitch filter width set to (%i + 1) * 2.\n",current_glitch_filter_width);            
    DRY_PGFR7 &= ~DRY_PGFR_GFW(0x3F);
    DRY_PGFR7 |= DRY_PGFR_GFW(desired_glitch_filter_width);
    printf("Tamper Pin 7 glitch filter width set to (%i + 1) * 2.\n",desired_glitch_filter_width);
    printf("DRY_PGFR7 is %08X\n",DRY_PGFR7);    
  }      
}
////////////////////////////////////////////////////////////////////////////////
void dryice_active_tamper_config(int active_tamper_reg, int desired_polynomial, int desired_shift_reg_value)
{
  int32 atr_reg_contents1,atr_reg_contents2,atr_reg_contents3;

  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Pin Active Tamper Config Utility\n");
  printf("-------------------------------------------\n");
  printf("\n");
  
  if ((DRY_CR & DRY_CR_DEN_MASK) != DRY_CR_DEN_MASK)
  {
    printf("DryIce prescaler and clock are disabled. Please enable them first before modifying the active tamper registers.\n\n");
    printf("\n");
    return;
  }  
  
  switch (active_tamper_reg)
  {
  case 0:
    if (((DRY_RAC & DRY_RAC_ATR(0x1))>>(DRY_RAC_ATR_SHIFT+0) == 0x0))
    {
      printf("Cannot read DRY_ATR0 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }    
    if (((DRY_WAC & DRY_WAC_ATW(0x1))>>(DRY_WAC_ATW_SHIFT+0) == 0x0))
    {
      printf("Cannot write DRY_ATR0 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_ATL(0x1))>>(DRY_LR_ATL_SHIFT+0) == 0x0))
    {
      printf("Cannot write DRY_ATR0 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }            
    printf("\nCurrent DRY_ATR0 is %08X\n",DRY_ATR0);
    atr_reg_contents1 = DRY_ATR_ATP(desired_polynomial);
    atr_reg_contents2 = DRY_ATR_ATSR(desired_shift_reg_value);
    atr_reg_contents3 = atr_reg_contents1 | atr_reg_contents2;    
    printf("Writing new values to DRY_ATR0.\n");    
    DRY_ATR0 = atr_reg_contents3;    
    printf("New DRY_ATR0 is %32b (%08X)\n",DRY_ATR0,DRY_ATR0);    
    break;

  case 1:
    if (((DRY_RAC & DRY_RAC_ATR(0x2))>>(DRY_RAC_ATR_SHIFT+1) == 0x0))
    {
      printf("Cannot read DRY_ATR1 because its read access is blocked. Remove the read access block and try again.\n");
      printf("DRY_RAC is \t%08X\n",DRY_RAC);    
      printf("\n");
      return;
    }
    if (((DRY_WAC & DRY_WAC_ATW(0x2))>>(DRY_WAC_ATW_SHIFT+1) == 0x0))
    {
      printf("Cannot write DRY_ATR1 because its write access is blocked. Remove the write access block and try again.\n");
      printf("DRY_WAC is \t%08X\n",DRY_WAC);    
      printf("\n");
      return;
    }
    if (((DRY_LR & DRY_LR_ATL(0x2))>>(DRY_LR_ATL_SHIFT+1) == 0x0))
    {
      printf("Cannot write DRY_ATR1 because it is locked. Remove the lock and try again.\n");
      printf("DRY_LR is \t%08X\n",DRY_LR);    
      printf("\n");
      return;
    }        
    printf("\n Current DRY_ATR1 is %08X\n",DRY_ATR1);
    atr_reg_contents1 = DRY_ATR_ATP(desired_polynomial);
    atr_reg_contents2 = DRY_ATR_ATSR(desired_shift_reg_value);
    atr_reg_contents3 = atr_reg_contents1 | atr_reg_contents2;    
    printf("Writing new values to DRY_ATR1.\n");    
    DRY_ATR1 = atr_reg_contents3;    
    printf("New DRY_ATR1 is %32b (%08X)\n",DRY_ATR1,DRY_ATR1);    
    break;
  }
  printf("\n");  
  return;
}// end dryice_active_tamper_config
////////////////////////////////////////////////////////////////////////////////
void dryice_display_active_tamper_regs(void)
{
  int shiftreg0;
  int shiftreg1;
  int active_tamper0_output, active_tamper1_output, xor_active_tamper_outputs;
  int cnt=1;
  int current_value;
  current_value = 0;
  
  printf("\n-------------------------------------------\n");
  printf("  DryIce Active Tamper Register Values\n");
  printf("-------------------------------------------\n");
  printf("\n");
  if ((DRY_CR & DRY_CR_DEN_MASK) != DRY_CR_DEN_MASK)
  {
    printf("DryIce prescaler and clock are disabled, so active tamper registers are static. Please enable DryIce prescaler and clock first.\n\n");
    printf("\n");
    return;
  }

  printf("\nPress any key to exit.\n\n");
  printf("-----Active Tamper 1 Output----------------------------------------+\n");
  printf("-----Active Tamper 0 Output----+                                   |\n");
  printf("                               |                                   |\n");
  printf("                               |                                   |\n");
  printf("                               V                                   V\n");
  while(1)
  {
    if(current_value != DRY_ATR0)
    {
      shiftreg0 = DRY_ATR0 & 0x0000FFFF;
      active_tamper0_output = (shiftreg0 & 0x01);

      shiftreg1 = DRY_ATR1 & 0x0000FFFF;
      active_tamper1_output = (shiftreg1 & 0x01);
      
      xor_active_tamper_outputs = active_tamper0_output ^ active_tamper1_output;      
      printf("%03i. ", cnt);
      printf("ATR0[ATSR]=%016b (%04X); ATR1[ATSR]=%016b (%04X); Active Tamper 0 output XOR Active Tamper 1 output: %1b\n",shiftreg0,shiftreg0,shiftreg1,shiftreg1,xor_active_tamper_outputs);      
      cnt+=1;
      current_value = DRY_ATR0;
    }
    if(char_present()){
      break;
    }
  }     
  printf("\n");  
  return;
}// end dryice_display_active_tamper_regs

////////////////////////////////////////////////////////////////////////////////

void dryice_enable_clock_and_prescaler(int desired_dryice_clock_switch_setting)
{
  int current_dryice_clock_switch_setting;

  
  printf("\n--------------------------------------------------------\n");
  printf("  DryIce Enable/Disable Prescaler and 32.768 kHz Clock \n");
  printf("--------------------------------------------------------\n");
  printf("\n");  
  


  
  if ((desired_dryice_clock_switch_setting < 0) || (desired_dryice_clock_switch_setting > 1))
  {
    printf("Invalid DryIce clock and prescaler option:\n");
    return;    
  }
  current_dryice_clock_switch_setting = (DRY_CR & DRY_CR_DEN_MASK);
  printf("Previous DryIce clock and prescaler setting was \t%i\n",current_dryice_clock_switch_setting>>DRY_CR_DEN_SHIFT);
  if (desired_dryice_clock_switch_setting == 0)
  {
    DRY_CR &= ~DRY_CR_DEN_MASK;
  }
  else
  {
    DRY_CR |= DRY_CR_DEN_MASK;
  }
  current_dryice_clock_switch_setting = (DRY_CR & DRY_CR_DEN_MASK);
  printf("New DryIce clock and prescaler setting is \t\t%i\n",current_dryice_clock_switch_setting>>DRY_CR_DEN_SHIFT);
  printf("\n");  
  return;  
}


