/*-------------------------------------------------------------------------*/
/*
 * \file    I2S,c
 * \brief   I2S initialization and interrupt service routine 
 *          for I2S peripheral
 *
 */
/*-------------------------------------------------------------------------*/
#include "sai.h"

#if SAI_RUN_UNDER_MQX != 0
  #include <mqx.h>
  #include <bsp.h>
#endif

#if SAI_RUN_UNDER_MQX == 0
  #error The Driver is not prepared to run without MQX!!!
#endif



typedef struct
{
  unsigned short samplerate;
  unsigned long masterClock;
  unsigned char bitClockDivider;
}SAMPLES_DESCR;

typedef signed short OUT_BUFF[SAI_BUFFER_SIZE];

#define I2S_BIT_DEPTH 32

#define MCLK_IX0 12288000
#define MCLK_IX1 11289600

#define DMA_MINORLOOP_CNT 4

#if SAI_RUN_UNDER_MQX != 0
  extern _lwmem_pool_id mp3DecoderpoolId;
  OUT_BUFF *outBuffer;
#else
  OUT_BUFF outBuffer[SAI_BUFFER_CNT_REAL];
#endif
int currBuffIx;
int lastBuffIx = -1;
unsigned long buffSize = 0; 
unsigned long buffSampleIx = 0;

static int sample_rate;

#if I2S_BIT_DEPTH == 16

const SAMPLES_DESCR samples_descr[] = 
{
  {8000, MCLK_IX0, 23},
  {11025, MCLK_IX1, 15},
  {12000, MCLK_IX0, 15},
  {16000, MCLK_IX0, 11},
  {22050, MCLK_IX1, 7},
  {24000, MCLK_IX0, 7},
  {32000, MCLK_IX0, 5},
  {44100, MCLK_IX1, 3},
  {48000, MCLK_IX0, 3},
  {0, 0, 0}
};
#elif I2S_BIT_DEPTH == 24
const SAMPLES_DESCR samples_descr[] = 
{
  {8000, MCLK_IX0, 15},
  {11025, MCLK_IX1, 10},
  {12000, MCLK_IX0, 10},
  {16000, MCLK_IX0, 7},
  {22050, MCLK_IX1, 5},
  {24000, MCLK_IX0, 5},
  {32000, MCLK_IX0, 3},
  {44100, MCLK_IX1, 2},
  {48000, MCLK_IX0, 2},
  {0, 0, 0}
};
#elif I2S_BIT_DEPTH == 32
const SAMPLES_DESCR samples_descr[] = 
{
  {8000, MCLK_IX0, 11},
  {11025, MCLK_IX1, 7},
  {12000, MCLK_IX0, 7},
  {16000, MCLK_IX0, 5},
  {22050, MCLK_IX1, 3},
  {24000, MCLK_IX0, 3},
  {32000, MCLK_IX0, 1},
  {44100, MCLK_IX1, 1},
  {48000, MCLK_IX0, 1},
  {0, 0, 0}
};
#else
#error Unsupported I2S_BIT_DEPTH!
#endif


static int SAI_GetSampleRateIx(int smprate);
void SAI_IRQHandler(pointer foo);
void SAI_DmaTxHandler(pointer foo);


/*-------------------------------------------------------------------------*/
/*!
 * \brief   Initialize I2S peripheral on Kinetis K40
 *
 */
/*-------------------------------------------------------------------------*/
void SAI_Init(void)
{
    /********************************/
    /*  Setup the I2S - data link   */
    /********************************/

    // enable system clock to the I2S module
    SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK;  
    SIM_SCGC6 |= SIM_SCGC6_SAI0_MASK | SIM_SCGC6_DMAMUX0_MASK;
    SIM_SCGC7 |= SIM_SCGC7_DMA_MASK;
    
    PORTE_PCR8 = PORT_PCR_MUX(1) | PORT_PCR_DSE_MASK;
    GPIOE_PDDR |= 1<<8;
    
    
    /* enable SSI pins */
    PORTE_PCR6 |= PORT_PCR_MUX(0x04);  // configure port for MCLK output
   
    //GPIO for SSI0_TX_BLCK
    PORTE_PCR12 |= PORT_PCR_MUX(0x04);  
     
    //GPIO for SSI0_TX_FS
    PORTE_PCR11 |= PORT_PCR_MUX(0x04);      
   
    //GPIO for SSI0_TXD
    PORTE_PCR10 |= PORT_PCR_MUX(0x04);
    
#if SAI_RUN_UNDER_MQX != 0
    
    outBuffer = (OUT_BUFF*)_mem_alloc_from(mp3DecoderpoolId, (sizeof(OUT_BUFF) * SAI_BUFFER_CNT_REAL));
    
    if(outBuffer == NULL)
      return;
    

#endif 
    
    
    // disable TX
    I2S_TCSR_REG(I2S0_BASE_PTR) &= ~I2S_TCSR_TE_MASK;
    I2S_TCR3_REG(I2S0_BASE_PTR) &= ~I2S_TCR3_TCE_MASK;
    I2S_TCSR_REG(I2S0_BASE_PTR) &= ~I2S_TCSR_BCE_MASK;
    
    // disable RX
    I2S_RCSR_REG(I2S0_BASE_PTR) &= ~I2S_RCSR_RE_MASK;
    I2S_RCR3_REG(I2S0_BASE_PTR) &= ~I2S_RCR3_RCE_MASK;
    I2S_RCSR_REG(I2S0_BASE_PTR) &= ~I2S_RCSR_BCE_MASK;
    
    I2S_MCR_REG(I2S0_BASE_PTR) |= I2S_MCR_MOE_MASK;
    
    // Set input clock to sysclock
    I2S_MCR_REG(I2S0_BASE_PTR) &= ~I2S_MCR_MICS_MASK;
    I2S_TCR2_REG(I2S0_BASE_PTR) = 0;
    
    sample_rate = 0;
    
    SAI_SetSampleRate(44100);

    // do sw reset
    I2S_TCSR_REG(I2S0_BASE_PTR) |= I2S_TCSR_SR_MASK;
    I2S_TCSR_REG(I2S0_BASE_PTR) &= ~I2S_TCSR_SR_MASK;
    
    // do fifo reset
    I2S_TCSR_REG(I2S0_BASE_PTR) |= I2S_TCSR_FR_MASK;
    
    // set fifo watermark
    I2S_TCR1_REG(I2S0_BASE_PTR) &= ~I2S_TCR1_TFW_MASK;
    I2S_TCR1_REG(I2S0_BASE_PTR) |= I2S_TCR1_TFW(4);
    
    // set the synch mode, direction and polarity 
    I2S_TCR2_REG(I2S0_BASE_PTR) |= ( I2S_TCR2_SYNC(0) | I2S_TCR2_BCD_MASK | I2S_TCR2_BCP_MASK | I2S_TCR2_MSEL(1));
    
    // configure frame size, sync width, assertion time, polarity and direction
    I2S_TCR4_REG(I2S0_BASE_PTR) = ( I2S_TCR4_FRSZ(1) | I2S_TCR4_SYWD(I2S_BIT_DEPTH - 1) | I2S_TCR4_FSE_MASK | I2S_TCR4_FSD_MASK | I2S_TCR4_MF_MASK);
    
    // configure the word 0 and subsequent word sizes with first bit shifted
    I2S_TCR5_REG(I2S0_BASE_PTR) = ( I2S_TCR5_WNW(I2S_BIT_DEPTH - 1) | I2S_TCR5_W0W(I2S_BIT_DEPTH - 1) | I2S_TCR5_FBT(0x0f) );
    
    // clear the receive and transmit mask registers
    I2S_TMR_REG(I2S0_BASE_PTR) = 0xFFFFFFFC;    
    

#if SAI_MODE == SAI_MODE_DMA    
    /* Set Up DMA channel*/
#if SAI_RUN_UNDER_MQX != 0
    // Install DMA0 Major loop complete IRQ  
    _int_install_isr(INT_DMA0_DMA16, SAI_DmaTxHandler, NULL);
    _cortex_int_init(INT_DMA0_DMA16, 3, TRUE);
#endif  
    
    DMA_ERQ &= ~DMA_ERQ_ERQ0_MASK;
    
    // Set up the DMA multiplexer
    DMAMUX0_CHCFG0 = DMAMUX_CHCFG_ENBL_MASK | DMAMUX_CHCFG_SOURCE(15); // I2S_DMA MUX trasmit
      
    DMA_CR = DMA_CR_GRP0PRI(1); //DMA_CR_EMLM_MASK;       // no minor loop mapping
    
    // fill the TCD area 
    DMA_TCD0_SADDR          = (uint32_t) outBuffer; // alternated with Audio_Source_Blk_B
    DMA_TCD0_SOFF           = DMA_MINORLOOP_CNT / 2;                              // 2 byte offset 
    DMA_TCD0_ATTR           = DMA_ATTR_SMOD(0) | DMA_ATTR_SSIZE(1) | DMA_ATTR_DMOD(0) | DMA_ATTR_DSIZE(1);   // no circular addressing S&D, 16 bit S&D 
    DMA_TCD0_NBYTES_MLNO    = DMA_MINORLOOP_CNT / 2;                              // one  16bit sample every minor loop 
    DMA_TCD0_SLAST          = 0;         // source address will continue X times to allow defined multiply length of buffer // to do
    DMA_TCD0_DADDR          = (uint32_t) &I2S_TDR_REG(I2S0_BASE_PTR, 0);    // the SAI Channel 0 duty value  
    DMA_TCD0_DOFF           = 0;
    DMA_TCD0_CITER_ELINKNO  = SAI_BUFFER_SIZE;              // total samples ( 128 ) // to do
    DMA_TCD0_DLASTSGA       = 0;                       // no final last adjustment ( does not move )
    DMA_TCD0_CSR            = DMA_CSR_INTMAJOR_MASK;    //DMA_CSR_INTMAJOR_MASK ;  // interrupt when done  
    DMA_TCD0_BITER_ELINKNO  = SAI_BUFFER_SIZE;              // no chan links, total samples ( 128 ) // to do
#elif SAI_MODE == SAI_MODE_INT
   
        //MQX: Install SAI0 IRQ
    _int_install_isr(INT_I2S0_Tx, SAI_IRQHandler, NULL);
    _cortex_int_init(INT_I2S0_Tx, 4, TRUE);
#endif   
    
    currBuffIx = 0;   
} /* end init() */

unsigned short* SAI_GetOutBuff(unsigned char ix)
{
  if(ix >= SAI_BUFFER_CNT_REAL)
    return NULL;
  
  return (unsigned short*)&outBuffer[ix];
}

unsigned long SAI_GetOutBuffCnt(void)
{
  return SAI_BUFFER_CNT_REAL;  
}

int SAI_EnableTx(unsigned long currBuffSize)
{
  if(currBuffSize > SAI_BUFFER_SIZE)
    return -1;
  
  currBuffIx = 0;
  
#if SAI_MODE == SAI_MODE_DMA  
  DMA_ERQ &= ~DMA_ERQ_ERQ0_MASK;
  
  DMA_TCD0_SADDR          = (uint32_t) outBuffer; // alternated with Audio_Source_Blk_B
  DMA_TCD0_SLAST          = (SAI_BUFFER_SIZE - currBuffSize) * 2;         // source address will continue X times to allow defined multiply length of buffer
  DMA_TCD0_CITER_ELINKNO  = (currBuffSize / 1);// / DMA_MINORLOOP_CNT;              // total samples ( 128 ) // to do
  DMA_TCD0_BITER_ELINKNO  = (currBuffSize / 1);// / DMA_MINORLOOP_CNT;              // no chan links, total samples ( 128 ) // to do
  
  I2S_TCSR_REG(I2S0_BASE_PTR) |= (I2S_TCSR_TE_MASK | I2S_TCSR_FWDE_MASK);
  I2S_TCR3_REG(I2S0_BASE_PTR) |= I2S_TCR3_TCE(1);
  
  // now enable chan0 for HW triggers
  DMA_ERQ |= DMA_ERQ_ERQ0_MASK;
#elif SAI_MODE == SAI_MODE_INT
  buffSize = currBuffSize; 
  
  I2S_TCSR_REG(I2S0_BASE_PTR) |= (I2S_TCSR_TE_MASK | I2S_TCSR_FRIE_MASK | I2S_TCSR_FWIE_MASK );
  I2S_TCR3_REG(I2S0_BASE_PTR) |= I2S_TCR3_TCE(1);
  
#endif  
  lastBuffIx = -1;
  return 0;
}

void SAI_PauseTx(void)
{
  I2S_TCR3_REG(I2S0_BASE_PTR) &= ~(I2S_TCR3_TCE(1));
  
#if SAI_MODE == SAI_MODE_DMA
  I2S_TCSR_REG(I2S0_BASE_PTR) &= ~(I2S_TCSR_TE_MASK | I2S_TCSR_BCE_MASK | I2S_TCSR_FWDE_MASK | I2S_TCSR_FRDE_MASK); 
#elif SAI_MODE == SAI_MODE_INT
  I2S_TCSR_REG(I2S0_BASE_PTR) &= ~(I2S_TCSR_TE_MASK | I2S_TCSR_BCE_MASK |  I2S_TCSR_FRIE_MASK); 
#endif  
  
}

void SAI_ResumeTx(void)
{
#if SAI_MODE == SAI_MODE_DMA  
  I2S_TCSR_REG(I2S0_BASE_PTR) |= (I2S_TCSR_TE_MASK | I2S_TCSR_FWDE_MASK);
#elif SAI_MODE == SAI_MODE_INT
  I2S_TCSR_REG(I2S0_BASE_PTR) |= (I2S_TCSR_TE_MASK | I2S_TCSR_FRIE_MASK);
#endif  
  I2S_TCR3_REG(I2S0_BASE_PTR) |= I2S_TCR3_TCE(1);
}

void SAI_DisableTx(void)
{
  I2S_TCR3_REG(I2S0_BASE_PTR) &= ~(I2S_TCR3_TCE(1));
  
#if SAI_MODE == SAI_MODE_DMA  
  I2S_TCSR_REG(I2S0_BASE_PTR) &= ~(I2S_TCSR_TE_MASK | I2S_TCSR_BCE_MASK | I2S_TCSR_FWDE_MASK | I2S_TCSR_FRDE_MASK);
#elif SAI_MODE == SAI_MODE_INT
  I2S_TCSR_REG(I2S0_BASE_PTR) &= ~(I2S_TCSR_TE_MASK | I2S_TCSR_BCE_MASK | I2S_TCSR_FRIE_MASK | I2S_TCSR_FWIE_MASK);
#endif
  
  buffSampleIx = 0;
  currBuffIx = 0;
}

void SAI_LastBlockLoaded(signed short* pBuffer)
{
  int i;
  for(i=0; i<SAI_BUFFER_CNT_REAL;i++)
  {
    if(outBuffer[i] == pBuffer)
    {
      lastBuffIx = i;
      break;
    } 
  }
  
}

static int SAI_GetSampleRateIx(int smprate)
{
  int ix = 0;    
  
  // go through the sample rate table
  while(samples_descr[ix].samplerate)
  {
    // if the requested sample rate is found, return the index (positive numer)
    if(samples_descr[ix].samplerate == smprate)
      return ix;
    
    ix++;
  } 
  return -1;
}

int SAI_SetSampleRate(int smprate)
{
   int srIx;
   
   if(sample_rate == smprate)
    return 0;
    sample_rate = smprate;
   
   srIx = SAI_GetSampleRateIx(smprate); 
    
   // Check if requested sample rate is supported
   if(srIx < 0)
     return -1;

  // disable MCLK output
//  I2S_MCR_REG(I2S0_BASE_PTR) &= ~I2S_MCR_MOE_MASK; 
   
  // Clear possible clock settings
  while(I2S_MCR_REG(I2S0_BASE_PTR) & I2S_MCR_DUF_MASK)
  {};
   I2S_MDR_REG(I2S0_BASE_PTR) = 0;
  
  while(I2S_MCR_REG(I2S0_BASE_PTR) & I2S_MCR_DUF_MASK)
  {};
  // get the 11.2896 MHz from 120MHz
  if(samples_descr[srIx].masterClock == MCLK_IX1)
    // Set the clocks - it should be 11.2896 MHz, and it is 11.2941MHz = 120MHz * (178 / 1892)
    I2S_MDR_REG(I2S0_BASE_PTR) =  I2S_MDR_FRACT(178 - 1) | I2S_MDR_DIVIDE(1892 - 1);   
  else  
    // Set the clocks - it should be 12.2880 MHz, and it is 12.2880MHz = 120MHz * (128 / 1250)
    I2S_MDR_REG(I2S0_BASE_PTR) =  I2S_MDR_FRACT(128 - 1) | I2S_MDR_DIVIDE(1250 - 1); 
  
  // bit clock divider (4 = (DIV + 1) * 2)
  I2S_TCR2_REG(I2S0_BASE_PTR) &= ~I2S_TCR2_DIV_MASK;
  I2S_TCR2_REG(I2S0_BASE_PTR) |= I2S_TCR2_DIV(samples_descr[srIx].bitClockDivider);    
  
  // enable MCLK output
  //I2S_MCR_REG(I2S0_BASE_PTR) |= I2S_MCR_MOE_MASK;
  
  while(I2S_MCR_REG(I2S0_BASE_PTR) & I2S_MCR_DUF_MASK)
  {};
  
  return 0;
}

void SAI_IRQHandler(pointer foo)
{
  int i;
  signed short sample;
  signed  long sampleFine;
  
  unsigned int tcsr = I2S_TCSR_REG(I2S0_BASE_PTR);	//transmit control register
  
  for(i=0; i<4;i++)
  {
    sample = outBuffer[currBuffIx][buffSampleIx++];
    sampleFine = sample;
    I2S_TDR_REG(I2S0_BASE_PTR, 0) = sampleFine;      
  }
  
  if(buffSampleIx > buffSize)
  { 
  #ifdef SAI_FREEBUFFER_EVENT_CB
    int freeBuffIx = currBuffIx;
  #endif
    
    buffSampleIx = 0;
    
  
    
    currBuffIx++;

    if(currBuffIx >= SAI_BUFFER_CNT_REAL)
      currBuffIx = 0;
    
  #ifdef SAI_FREEBUFFER_EVENT_CB
    SAI_FREEBUFFER_EVENT_CB(freeBuffIx);    
  #endif
  }
 I2S_TCSR_REG(I2S0_BASE_PTR) |= I2S_TCSR_WSF_MASK | I2S_TCSR_SEF_MASK | I2S_TCSR_FEF_MASK; 
}



void SAI_DmaTxHandler(pointer foo)
{
  #ifdef SAI_FREEBUFFER_EVENT_CB
    int freeBuffIx = currBuffIx;
  #endif  
  
  currBuffIx++;


    
  
  if(currBuffIx >= SAI_BUFFER_CNT_REAL)
  {
     DMA_TCD0_SADDR = (uint32_t) outBuffer; // alternated with Audio_Source_Blk_B  
     currBuffIx = 0;
  }
  if(lastBuffIx < 0)
  {  
#ifdef SAI_FREEBUFFER_EVENT_CB
    SAI_FREEBUFFER_EVENT_CB(freeBuffIx);    
#endif  
  }else
  {
    if(lastBuffIx == freeBuffIx)
      SAI_DisableTx();
  }
  DMA_CINT = DMA_CINT_CINT(0);                 // use the Clear Intr. Request register 
}
